﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using StrategyInterface;
using System.Drawing;

namespace Levels_SAR
{
    /// <summary>
    /// Торговая стратегия Levels_SAR
    /// </summary>
    public class Levels_SAR : StrategyBase, IStrategy
    {
        #region Конструктор

        public Levels_SAR()
        {
            // Создаем группу элементов с идентификаторами графиков
            Controls.Add("Group1", new S_GroupBox("Идентификаторы графиков"));
            // Поля для задания идентификаторов графика баров и индикаторов
            Controls.Add("Price", new S_GraphID_TextBox() { LabelText = "Идентификатор графика цены:", Text = "PRICE", Width = 152, XOffset = -40, Itype = IndicatorType.Price });
            Controls.Add("SAR",   new S_GraphID_TextBox() { LabelText = "Идентификатор Parabolic SAR:", Text = "SAR", Width = 152, XOffset = -40, Itype = IndicatorType.ParabolicSAR });
            // Сигналы на открытие и закрытие позиции
            Controls.Add("Group2", new S_GroupBox("Сигналы"));
            Controls.Add("Position_open",  new S_ComboBox() { LabelText = "Сигнал открытия позиции:", Width = 122, XOffset = -10, Items = new string[] { "Нет", "По пробою", "С подтверждением", "По проколу", "По Parabolic SAR" }, SelectedItem = 0, AllowEdit = false });
            Controls.Add("Position_close", new S_ComboBox() { LabelText = "Сигнал закрытия позиции:", Width = 122, XOffset = -10, Items = new string[] { "Нет", "По пробою", "С подтверждением", "По проколу", "По Parabolic SAR" }, SelectedItem = 0, AllowEdit = false });
            // Дополнительные условия механической торговой системы
            Controls.Add("Group3", new S_GroupBox("Дополнительные условия"));
            Controls.Add("Type_of_channel",    new S_ComboBox()      { LabelText = "Формирование уровней:", Width = 112, Items = new string[] { "Из мин и макс", "Из цен закрытий", "Из октр и закр" }, SelectedItem = 0, AllowEdit = false });
            Controls.Add("Long_period_open",   new S_NumericUpDown() { LabelText = "Кол-во свечей для входа в Лонг:",    Width = 60, XOffset = 52, Min = 1, Max = 100, Value = 20 });
            Controls.Add("Long_period_close",  new S_NumericUpDown() { LabelText = "Кол-во свечей для выхода из Лонга:", Width = 60, XOffset = 52, Min = 1, Max = 100, Value = 10 });
            Controls.Add("Short_period_open",  new S_NumericUpDown() { LabelText = "Кол-во свечей для входа в Шорт:",    Width = 60, XOffset = 52, Min = 1, Max = 100, Value = 20 });
            Controls.Add("Short_period_close", new S_NumericUpDown() { LabelText = "Кол-во свечей для выхода из Шорта:", Width = 60, XOffset = 52, Min = 1, Max = 100, Value = 10 });
            Controls.Add("Levels_correct",     new S_TextBox()       { LabelText = "Дельта коррекции уровней, пункты:", Text = "0", Width = 60, XOffset = 52, ValueType = TextBoxValueType.Numeric });
            Controls.Add("Close_by_SAR",       new S_CheckBox("Выход по пересечению Parabolic SAR", false));
            Controls.Add("SAR_current_trade",  new S_CheckBox("Торговля по текущим значениям Parabolic SAR", true));
            // Последняя группа элементов
            Controls.Add("Group4", new S_GroupBox("Информационные поля"));
            // Поля для вывода информации о используемых в работе значений
            Controls.Add("longOpen",   new S_InfoBox() { LabelText = "Вход в Лонг:", Width = 112 });
            Controls.Add("longClose",  new S_InfoBox() { LabelText = "Выход из Лонга:", Width = 112 });
            Controls.Add("shortOpen",  new S_InfoBox() { LabelText = "Вход в Шорт:", Width = 112 });
            Controls.Add("shortClose", new S_InfoBox() { LabelText = "Выход из Шорта:", Width = 112 });
            Controls.Add("sarValue",   new S_InfoBox() { LabelText = "Значение Parabolic SAR:", Width = 112 });
        }

        #endregion

        #region Функции обратного вызова

        /// <summary>
        /// Вызывается при изменении цены последней сделки
        /// </summary>
        /// <param name="price">Цена последней сделки</param>
        public override void OnPrice(decimal price)
        {
            Main();
        }

        /// <summary>
        /// Выполняется с заданным интервалом (по умолчанию 1 сек)
        /// </summary>
        public override void OnTimer()
        {
            Main();
        }

        #endregion

        #region Main()

        // Флаги подсветки полей ввода идентификаторов индикаторов
        bool sar_is_error   = false;
        bool price_is_error = false;

        private void Main()
        {
            // Очищаем ошибки предыдущей итерации
            Log.LastError = string.Empty;

            // Получаем информацию о бумаге
            ISecurityInfo SecInfo = Data.getSecurityInfo();
            if (SecInfo == null)
            {
                Log.Error(string.Format("Не удалось получить информацию по бумаге {0}/{1}, проверьте параметры инструмента",
                                Settings.SecCode, Settings.ClassCode));
                return;
            }

            // Получаем цены
            IPriceInfo Prices = Data.getPrices();
            if (Prices == null || Prices.LastPrice == 0)
            {
                Log.Error(string.Format("Не удалось получить информацию по ценам для {0}/{1}, проверьте параметры инструмента",
                                Settings.SecCode, Settings.ClassCode));
                return;
            }

            // Получаем заданные периоды каналов
            int Long_period_open   = (int)Data.getControlValue<decimal>("Long_period_open");
            int Long_period_close  = (int)Data.getControlValue<decimal>("Long_period_close");
            int Short_period_open  = (int)Data.getControlValue<decimal>("Short_period_open");
            int Short_period_close = (int)Data.getControlValue<decimal>("Short_period_close");
            int MaxPeriods = Math.Max(Math.Max(Math.Max(Long_period_open, Long_period_close), Short_period_open), Short_period_close) + 2;

            // Получаем свечи ценового графика
            string PriceIndID = Data.getControlValue<string>("Price");
            string get_candles_error = string.Empty;
            ICandleInfo[] Candles = Data.getPriceCandles(PriceIndID, MaxPeriods, out get_candles_error);

            // Идентификатор SAR
            string SAR_name = Data.getControlValue<string>("SAR");

            string get_SAR_error = string.Empty;
            IIndicatorValueEx[] SAR_data = Data.getIndicatorValuesEx(SAR_name, 2, out get_SAR_error);
            if (get_SAR_error != string.Empty)
            {
                if (!sar_is_error)
                {
                    sar_is_error = true;
                    Data.setControlColor("SAR", Color.Yellow, Color.Red);
                }
                Log.Error(get_SAR_error);
                return;
            }
            else if (sar_is_error)
            {
                Data.setControlColor("SAR", SystemColors.Window, SystemColors.WindowText);
            }

            if (get_candles_error != string.Empty)
            {
                if (!price_is_error)
                {
                    price_is_error = true;
                    Data.setControlColor("Price", Color.Yellow, Color.Red);
                }
                Log.Error(get_candles_error); //не удалось получить свечи
                return;
            }
            else if (price_is_error)
            {
                Data.setControlColor("Price", SystemColors.Window, SystemColors.WindowText);
            }

            // Проверка входных данных
            if (!CheckData(Candles, Prices, MaxPeriods, SAR_data)) return;

            // Очищаем информацию об ошибках
            Log.ResetError();

            // Выводим данные в информационные поля о индикаторах --------------
            Data.setInfoBoxValue("sarValue", SAR_data[0][0].ToString("F6"));

            // Получаем сигнал стратегии и передаем его в модуль торговли
            Trade.SetSignal(GetSignal(Data.getPrices(), Data.getPosition(), Candles, new decimal[] { SAR_data[0][0], SAR_data[1][0] }));
        }

        #endregion

        #region GetSignal

        private StrategySignal GetSignal(IPriceInfo Prices, IPositionInfo Position, ICandleInfo[] Candles, decimal[] SAR)
        {
            // Получаем значения контролов
            int     Position_open      = Data.getControlValue<int>("Position_open");
            int     Position_close     = Data.getControlValue<int>("Position_close");
            int     Long_period_open   = (int)Data.getControlValue<decimal>("Long_period_open");
            int     Long_period_close  = (int)Data.getControlValue<decimal>("Long_period_close");
            int     Short_period_open  = (int)Data.getControlValue<decimal>("Short_period_open");
            int     Short_period_close = (int)Data.getControlValue<decimal>("Short_period_close");
            int     Type_of_channel    = Data.getControlValue<int>("Type_of_channel");
            decimal Levels_correct     = Helper.toDecimal(Data.getControlValue <string>("Levels_correct"));
            bool    Close_by_SAR       = Data.getControlValue<bool>("Close_by_SAR");
            bool    SAR_current_trade  = Data.getControlValue<bool>("SAR_current_trade");

            OpenSignal Open = OpenSignal.None;

            bool LongClose  = false;
            bool ShortClose = false;

            // Проверяем вид торговли: по текущим или предыдущим значениям
            int sar_index = (SAR_current_trade) ? 0 : 1;

            // Условия закрытия позиции по пересечению Parabolic SAR ----------------------
            if (Close_by_SAR)
            {
                if (Candles[0].Close < SAR[sar_index]) LongClose = true;
                if (Candles[0].Close > SAR[sar_index]) ShortClose = true;
                if (Position.Quantity > 0 && Candles[0].Close < SAR[sar_index]) return new StrategySignal(Open, true, false);
                if (Position.Quantity < 0 && Candles[0].Close > SAR[sar_index]) return new StrategySignal(Open, false, true);
            }

            // Определяем границы каналов для открытия и закрытия позиции -----------------
            decimal Level_long_open  = Candles[1].High;
            decimal Level_short_open = Candles[1].Low;
            if (Type_of_channel == 1)
            {
                Level_long_open  = Candles[1].Close;
                Level_short_open = Candles[1].Close;
            }
            if (Type_of_channel == 2)
            {
                Level_long_open  = Math.Max(Candles[1].Close, Candles[1].Open);
                Level_short_open = Math.Min(Candles[1].Close, Candles[1].Open);
            }
            decimal Level_short_close = Level_long_open;
            decimal Level_long_close  = Level_short_open;

            /*
            // Если способы открытия или закрытия не по пробою, то ------------------------
            if (Position_open > 0 || Position_close > 0)
            {
                if (!Type_of_channel)
                {
                    Level_long_open = CandlesList[2].HIGH;
                    Level_short_open = CandlesList[2].LOW;
                }
                else
                {
                    Level_long_open = CandlesList[2].CLOSE;
                    Level_short_open = CandlesList[2].CLOSE;
                }
            }
            */
            
            // Если способы открытия или закрытия не по пробою, то ------------------------
            int x_count = 1;
            if (Position_open > 1 || Position_close > 1)
            {
                x_count = 2;
            }

            List<ICandleInfo> CandlesList = Candles.ToList();

            // Выбираем наибольший максимум для открытия Лонга ----------------------------
            if (Long_period_open > 1)
            {
                switch (Type_of_channel)
                {
                    case 0: // Из мин и макс
                        Level_long_open = CandlesList.GetRange(x_count, Long_period_open).Max(a => a.High);
                        break;
                    case 1: // Из цен закрытий
                        Level_long_open = CandlesList.GetRange(x_count, Long_period_open).Max(a => a.Close);
                        break;
                    case 2: // Из откр и закр
                        Level_long_open = CandlesList.GetRange(x_count, Long_period_open).Max(a => a.Close);
                        Level_long_open = Math.Max(Level_long_open, CandlesList.GetRange(x_count, Long_period_open).Max(a => a.Open));
                        break;
                }
            }

            // Выбираем наименьший минимум для закрытия Лонга -----------------------------
            if (Long_period_close > 1)
            {
                switch (Type_of_channel)
                {
                    case 0: // Из мин и макс
                        Level_long_close = CandlesList.GetRange(x_count, Long_period_close).Min(a => a.Low);
                        break;
                    case 1: // Из цен закрытий
                        Level_long_close = CandlesList.GetRange(x_count, Long_period_close).Min(a => a.Close);
                        break;
                    case 2: // Из откр и закр
                        Level_long_close = CandlesList.GetRange(x_count, Long_period_close).Min(a => a.Close);
                        Level_long_close = Math.Min(Level_long_close, CandlesList.GetRange(x_count, Long_period_close).Min(a => a.Open));
                        break;
                }
            }

            // Выбираем наименьший минимум для открытия Шорта -----------------------------
            if (Short_period_open > 1)
            {
                switch (Type_of_channel)
                {
                    case 0: // Из мин и макс
                        Level_short_open = CandlesList.GetRange(x_count, Short_period_open).Min(a => a.Low);
                        break;
                    case 1: // Из цен закрытий
                        Level_short_open = CandlesList.GetRange(x_count, Short_period_open).Min(a => a.Close);
                        break;
                    case 2: // Из откр и закр
                        Level_short_open = CandlesList.GetRange(x_count, Short_period_open).Min(a => a.Close);
                        Level_short_open = Math.Min(Level_short_open, CandlesList.GetRange(x_count, Short_period_open).Min(a => a.Open));
                        break;
                }
            }

            // Выбираем наибольший максимум для закрытия Шорта ----------------------------
            if (Short_period_close > 1)
            {
                switch (Type_of_channel)
                {
                    case 0: // Из мин и макс
                        Level_short_close = CandlesList.GetRange(x_count, Short_period_close).Max(a => a.High);
                        break;
                    case 1: // Из цен закрытий
                        Level_short_close = CandlesList.GetRange(x_count, Short_period_close).Max(a => a.Close);
                        break;
                    case 2: // Из откр и закр
                        Level_short_close = CandlesList.GetRange(x_count, Short_period_close).Max(a => a.Close);
                        Level_short_close = Math.Max(Level_short_close, CandlesList.GetRange(x_count, Short_period_close).Max(a => a.Open));
                        break;
                }
            }

            // Выводим необходимые данные в информационные поля о индикаторах --------------
            Data.setInfoBoxValue("longOpen",   Level_long_open.ToString());
            Data.setInfoBoxValue("longClose",  Level_long_close.ToString());
            Data.setInfoBoxValue("shortOpen",  Level_short_open.ToString());
            Data.setInfoBoxValue("shortClose", Level_short_close.ToString());

            // ********************************
            // Проверяем на существование и корректность значений каналов
            if (Level_long_open > 0 && Level_long_close > 0 && Level_short_open > 0 && Level_short_close > 0)
            {
                // -----------------------------
                // Условия для ОТКРЫТИЯ позиций
                // -----------------------------
                switch (Position_open)
                {
                    case 0: // Нет
                        break;
                    case 1: // Открытие по пробою
                        if (Candles[0].Close > Level_long_open + Levels_correct)
                            Open = OpenSignal.Long;
                        else if (Candles[0].Close < Level_short_open - Levels_correct)
                            Open = OpenSignal.Short;
                        break;
                    case 2: // Открытие с подтверждением
                        if (Candles[1].Close > Level_long_open + Levels_correct)
                            Open = OpenSignal.Long;
                        else if (Candles[1].Close < Level_short_open - Levels_correct)
                            Open = OpenSignal.Short;
                        break;
                    case 3: // Открытие по проколу
                        if (Candles[1].High > Level_long_open + Levels_correct)
                            Open = OpenSignal.Long;
                        else if (Candles[1].Low < Level_short_open - Levels_correct)
                            Open = OpenSignal.Short;
                        break;
                    case 4: // Открытие по Parabolic SAR
                        if (Candles[0].Close > SAR[sar_index])
                            Open = OpenSignal.Long;
                        else if (Candles[0].Close < SAR[sar_index])
                            Open = OpenSignal.Short;
                        break;
                }

                // -----------------------------
                // Условия для ЗАКРЫТИЯ позиций
                // -----------------------------
                switch (Position_close)
                {
                    case 0: // Нет
                        break;
                    case 1: // Закрытие по пробою
                        if (Candles[0].Close < Level_long_close)
                            LongClose = true;
                        if (Candles[0].Close > Level_short_close)
                            ShortClose = true;
                        break;
                    case 2: // Закрытие с подтверждением
                        if (Candles[1].Close < Level_long_close)
                            LongClose = true;
                        if (Candles[1].Close > Level_short_close)
                            ShortClose = true;
                        break;
                    case 3: // Закрытие по проколу
                        if (Candles[1].Low < Level_long_close)
                            LongClose = true;
                        if (Candles[1].High > Level_short_close)
                            ShortClose = true;
                        break;
                    case 4: // Закрытие по Parabolic SAR
                        if (Candles[0].Close < SAR[sar_index])
                            LongClose = true;
                        if (Candles[0].Close > SAR[sar_index])
                            ShortClose = true;
                        break;
                }
                // -----------------------------
            }

            return new StrategySignal(Open, LongClose, ShortClose);
        }

        #endregion

        #region Проверка данных

        // Проверка данных
        private bool CheckData(ICandleInfo[] Candles, IPriceInfo Prices, int MaxPeriods, IIndicatorValueEx[] SAR)
        {
            // Проверяем значение цены последней сделки
            if (Prices.LastPrice <= 0)
            {
                Log.Error(string.Format("Неверное значение цены последней сделки {0}. Проверьте параметры инструмента.", 
                    Prices.LastPrice));
                return false;
            }

            // Проверяем значения цен баров
            if (Candles.Length < MaxPeriods)
            {
                Log.Error(string.Format("Недостаточное количество свечей. Проверьте настройки графика цены. (свечи: {0})", 
                    Candles.Length));
                return false;
            }
            if (Candles[0].Open <= 0 || Candles[1].Close <= 0 || Candles[2].Close <= 0)
            {
                Log.Error(string.Format("Неверные значения цен свечей. Проверьте настройки графика цены. ({0} {1} {2})",
                    Candles[0].Open, Candles[1].Close, Candles[2].Close));
                return false;
            }

            int TimeFrame = Helper.GetTimeFrame(Candles);

            // Проверяем значение временного интервала баров
            if (TimeFrame == 0)
            {
                Log.Error("Таймфрейм равен 0. Проверьте настройки графика свечей.");
                return false;
            }

            // Проверяем значения индикаторов
            if (SAR.Length < 2 || SAR.Any(a => a.Length != 1))
            {
                Log.Error(string.Format("Недостаточное количество значений Parabolic SAR. Проверьте настройки индикатора. " +
                    "(свечи: {0}, линии: {1})", SAR.Length, SAR.Length > 0 ? SAR[0].Length : 0));
                return false;
            }
            if (SAR.Any(a => a.Values[0].Value == 0))
            {
                Log.Error(string.Format("Неверные значения Parabolic SAR. Проверьте настройки индикатора. ({0} {1})",
                    SAR[0][0], SAR[1][0]));
                return false;
            }

            return true;
        }

        #endregion
    }
}
